"""Utility functions for niconico module."""

from __future__ import annotations

import re


def extract_video_id_from_url(url: str) -> str | None:
    """Extract video ID from URL.

    Args:
        url (str): URL to extract video ID from.

    Returns:
        str | None: Extracted video ID or None if not found.
    """
    match = re.search(r"(?:sm|nm|so)?\d+", url)
    if match:
        return match.group(0)
    return None



"""This module provides a client for watching videos on Niconico."""

from __future__ import annotations

import asyncio
import json
import re
import secrets
import string
import subprocess
import time
from pathlib import Path

import requests

from niconico.base.client import BaseClient
from niconico.decorators import login_required
from niconico.exceptions import CommentAPIError, DownloadError, NicoAPIError, WatchAPIError
from niconico.objects.nvapi import AccessRightsData, NvAPIResponse, ThreadKeyData
from niconico.objects.video.watch import (
    NvCommentAPIData,
    NvCommentAPIResponse,
    StoryboardResponse,
    WatchAPIData,
    WatchAPIErrorData,
    WatchAPIResponse,
    WatchData,
)


class VideoWatchClient(BaseClient):
    """A client for watching videos on Niconico."""

    def get_watch_data(self, video_id: str) -> WatchData:
        """Get the watch data of a video.

        Args:
            video_id: The ID of the video.

        Returns:
            WatchData | WatchResponseError: The watch data of the video if successful, WatchResponseError
        """
        res = self.niconico.get(f"https://www.nicovideo.jp/watch/{video_id}?responseType=json")
        if res.status_code == requests.codes.ok:
            res_cls_data = WatchAPIResponse[WatchAPIData](**res.json())
            return res_cls_data.data.response
        res_cls_error = WatchAPIResponse[WatchAPIErrorData](**res.json())
        raise WatchAPIError(response=res_cls_error.data.response)

    def generate_action_track_id(self) -> str:
        """Generate a random action track ID.

        Returns:
            str: The generated action track ID.
        """
        fh_chars = string.ascii_letters + string.digits
        fh = "".join(secrets.choice(fh_chars) for _ in range(10))
        lh = int(time.time() * 1000)
        return f"{fh}_{lh}"

    def get_outputs(self, watch_data: WatchData, *, audio_only: bool = False) -> dict[str, list[str]]:
        """Get the outputs of a video.

        Args:
            watch_data: The watch data of the video.
            audio_only: Whether to get the audio only.

        Returns:
            dict[str, list[str]]: The outputs of the video.
        """
        outputs: dict[str, list[str]] = {}
        top_audio_id = None
        top_audio_quality = -1
        for audio in watch_data.media.domand.audios:
            if audio.is_available and audio.quality_level > top_audio_quality:
                top_audio_id = audio.id_
                top_audio_quality = audio.quality_level
        if top_audio_id is None:
            return outputs
        for video in watch_data.media.domand.videos:
            if video.is_available:
                outputs[video.label] = [top_audio_id] if audio_only else [video.id_, top_audio_id]
        return outputs

    def get_hls_content_url(self, watch_data: WatchData, outputs: list[list[str]]) -> str | None:
        """Get the HLS content URL of a video.

        Args:
            watch_data: The watch data of the video.
            outputs: The outputs. e.g.: [video_id, audio_id][]

        Returns:
            str | None: The HLS content URL of the video if successful, None otherwise.
        """
        video_id = watch_data.client.watch_id
        action_track_id = watch_data.client.watch_track_id
        access_right_key = watch_data.media.domand.access_right_key
        res = self.niconico.post(
            f"https://nvapi.nicovideo.jp/v1/watch/{video_id}/access-rights/hls?actionTrackId={action_track_id}",
            json={"outputs": outputs},
            headers={"X-Access-Right-Key": access_right_key},
        )
        if res.status_code == requests.codes.created:
            res_cls = NvAPIResponse[AccessRightsData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.content_url
        return None

    @login_required(premium=True)
    def get_storyboard_url(self, watch_data: WatchData) -> str | None:
        """Get the storyboards URL of a video.

        Args:
            watch_data: The watch data of the video.

        Returns:
            str | None: The storyboards URL of the video if successful, None otherwise.
        """
        if not watch_data.media.domand.is_storyboard_available:
            return None
        video_id = watch_data.client.watch_id
        action_track_id = watch_data.client.watch_track_id
        access_right_key = watch_data.media.domand.access_right_key
        res = self.niconico.post(
            f"https://nvapi.nicovideo.jp/v1/watch/{video_id}/access-rights/storyboard?actionTrackId={action_track_id}",
            headers={"X-Access-Right-Key": access_right_key},
        )
        if res.status_code == requests.codes.created:
            res_cls = NvAPIResponse[AccessRightsData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.content_url
        return None

    @login_required(premium=True)
    def download_storyboards(self, watch_data: WatchData, output_path: str) -> str:
        """Download the storyboards of a video.

        Args:
            watch_data: The watch data of the video.
            output_path: The folder path to save the storyboards.

        Returns:
            str: The path of the downloaded storyboards.
        """
        storyboard_url = self.get_storyboard_url(watch_data)
        if storyboard_url is None:
            raise NicoAPIError(message="Failed to get the storyboards URL.")
        output_path = output_path % {
            "id": watch_data.video.id_,
            "title": watch_data.video.title,
            "owner": watch_data.owner.nickname,
            "owner_id": str(watch_data.owner.id_),
            "timestamp": str(int(time.time())),
        }
        res = self.niconico.get(storyboard_url)
        if res.status_code == requests.codes.ok:
            res_cls = StoryboardResponse(**res.json())
            if not Path(output_path).is_dir():
                Path(output_path).mkdir(parents=True)
            if Path(f"{output_path}/storyboard.json").exists():
                raise DownloadError(message="The storyboard.json file already exists.")
            with Path(f"{output_path}/storyboard.json").open(mode="w") as f:
                f.write(res.text)
            for image in res_cls.images:
                image_res = self.niconico.get(re.sub(r"(?<=/)[^/]+(?=\?)", image.url, storyboard_url))
                if image_res.status_code == requests.codes.ok:
                    if Path(f"{output_path}/{image.url}.jpg").exists():
                        raise DownloadError(message=f"The storyboard image already exists: {image.url}")
                    with Path(f"{output_path}/{image.url}.jpg").open(mode="wb") as f:
                        f.write(image_res.content)
                else:
                    raise DownloadError(message=f"Failed to download the storyboard image: {image.url}")
        else:
            raise DownloadError(message="Failed to download the storyboards.")
        return output_path

    def download_video(
        self,
        watch_data: WatchData,
        output_label: str,
        output_path: str = "%(title)s.%(ext)s",
        *,
        audio_only: bool = False,
    ) -> str:
        """Download a video.

        Args:
            watch_data: The watch data of the video.
            output_label: The output label of the video.
            output_path: The path to save the video.
            audio_only: Whether to download the audio only.

        Returns:
            str: The path of the downloaded video.
        """
        outputs = self.get_outputs(watch_data, audio_only=audio_only)
        if output_label not in outputs:
            raise DownloadError(message="The output label is not available.")
        hls_content_url = self.get_hls_content_url(watch_data, [outputs[output_label]])
        if hls_content_url is None:
            raise NicoAPIError(message="Failed to get the HLS content URL.")
        output_path = output_path % {
            "id": watch_data.video.id_,
            "title": watch_data.video.title,
            "owner": watch_data.owner.nickname,
            "owner_id": str(watch_data.owner.id_),
            "timestamp": str(int(time.time())),
            "ext": "m4a" if audio_only else "mp4",
        }
        if not Path(output_path).parent.exists():
            Path(output_path).parent.mkdir(parents=True)
        if Path(output_path).exists():
            raise DownloadError(message="The video file already exists.")
        cookies = {
            "domand_bid": self.niconico.session.cookies.get("domand_bid"),
        }
        commands = " ".join(
            [
                "ffmpeg",
                "-headers",
                f"'cookie: {';'.join(f'{k}={v}' for k, v in cookies.items())}'",
                "-protocol_whitelist",
                "file,http,https,tcp,tls,crypto",
                "-i",
                f"'{hls_content_url}'",
                "-c",
                "copy",
                f"'{output_path}'",
            ],
        )
        try:
            with subprocess.Popen(  # noqa: S602
                commands,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                shell=True,
            ) as p:
                for line in iter(p.stdout.readline, b""):  # type: ignore[union-attr]
                    self.log("debug", line.rstrip())
                p.wait()
        except subprocess.CalledProcessError as e:
            raise DownloadError(message="Failed to download the video.") from e
        return output_path

    async def download_video_async(
        self,
        watch_data: WatchData,
        output_label: str,
        output_path: str = "%(title)s.%(ext)s",
        *,
        audio_only: bool = False,
    ) -> str:
        """Asynchronously download a video.

        Args:
            watch_data: The watch data of the video.
            output_label: The output label of the video.
            output_path: The path to save the video.
            audio_only: Whether to download the audio only.

        Returns:
            str: The path of the downloaded video.
        """
        outputs = self.get_outputs(watch_data, audio_only=audio_only)
        if output_label not in outputs:
            raise DownloadError(message="The output label is not available.")
        hls_content_url = self.get_hls_content_url(watch_data, [outputs[output_label]])
        if hls_content_url is None:
            raise NicoAPIError(message="Failed to get the HLS content URL.")
        output_path = output_path % {
            "id": watch_data.video.id_,
            "title": watch_data.video.title,
            "owner": watch_data.owner.nickname,
            "owner_id": str(watch_data.owner.id_),
            "timestamp": str(int(time.time())),
            "ext": "m4a" if audio_only else "mp4",
        }
        if not Path(output_path).parent.exists():
            Path(output_path).parent.mkdir(parents=True)
        if Path(output_path).exists():
            raise DownloadError(message="The video file already exists.")
        cookies = {
            "domand_bid": self.niconico.session.cookies.get("domand_bid"),
        }
        commands = " ".join(
            [
                "ffmpeg",
                "-headers",
                f"'cookie: {';'.join(f'{k}={v}' for k, v in cookies.items())}'",
                "-protocol_whitelist",
                "file,http,https,tcp,tls,crypto",
                "-i",
                f"'{hls_content_url}'",
                "-c",
                "copy",
                f"'{output_path}'",
            ],
        )
        try:
            process = await asyncio.create_subprocess_shell(
                commands,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.STDOUT,
            )
            async for line in process.stdout:  # type: ignore[union-attr]
                self.log("debug", line.decode().rstrip())
            await process.wait()
        except Exception as e:
            raise DownloadError(message="Failed to download the video.") from e
        return output_path

    def get_thread_key(self, video_id: str) -> str | None:
        """Get the thread key of a video.

        Args:
            video_id: The ID of the video.

        Returns:
            str: The thread key of the video.
        """
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/comment/keys/thread?videoId={video_id}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[ThreadKeyData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.thread_key
        return None

    def get_comments(
        self,
        watch_data: WatchData,
        *,
        when: int | None = None,
        thread_key: str | None = None,
    ) -> NvCommentAPIData | None:
        """Get the comments of a video.

        Args:
            watch_data: The watch data of the video.
            when: The time to get the comments.
            thread_key: The thread key of the video.

        Returns:
            list[Any]: The comments of the video.
        """
        payload = {
            "threadKey": watch_data.comment.nv_comment.thread_key,
            "params": watch_data.comment.nv_comment.params.model_dump(by_alias=True),
            "additionals": {},
        }
        if when is not None:
            if self.niconico.premium:
                payload["additionals"] = {"when": when}
            else:
                raise NicoAPIError(message="You must be a premium member to get the comments at a specific time.")
        if thread_key is not None:
            payload["threadKey"] = thread_key
        res = self.niconico.post(
            watch_data.comment.nv_comment.server + "/v1/threads",
            data=json.dumps(payload),
        )
        res_cls = NvCommentAPIResponse(**res.json())
        if res_cls.meta.status == requests.codes.ok:
            return res_cls.data
        raise CommentAPIError(message=res_cls.meta.error_code)



"""This module contains the class that represents a video object."""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field

from niconico.objects.common import EssentialChannel, EssentialUser


class VideoCount(BaseModel):
    """A class that represents the count of a video."""

    view: int
    comment: int
    mylist: int
    like: int


class VideoThumbnail(BaseModel):
    """A class that represents the thumbnail of a video."""

    url: str
    middle_url: str | None = Field(..., alias="middleUrl")
    large_url: str | None = Field(..., alias="largeUrl")
    listing_url: str = Field(..., alias="listingUrl")
    nhd_url: str = Field(..., alias="nHdUrl")


class Owner(BaseModel):
    """A class that represents the owner of a video."""

    owner_type: Literal["user", "channel", "hidden"] = Field(..., alias="ownerType")
    type_: Literal["user", "channel", "unknown"] = Field(..., alias="type")
    visibility: Literal["visible", "hidden"]
    id_: str | None = Field(..., alias="id")
    name: str | None
    icon_url: str | None = Field(..., alias="iconUrl")


class VideoLive(BaseModel):
    """A class that represents a live video."""

    live_start_time: str | None = Field(..., alias="liveStartTime")


class EssentialVideo(BaseModel):
    """A class that represents an essential video object."""

    type_: Literal["essential"] = Field(..., alias="type")
    id_: str = Field(..., alias="id")
    title: str
    registered_at: str = Field(..., alias="registeredAt")
    count: VideoCount
    thumbnail: VideoThumbnail
    duration: int
    short_description: str = Field(..., alias="shortDescription")
    latest_comment_summary: str = Field(..., alias="latestCommentSummary")
    is_channel_video: bool = Field(..., alias="isChannelVideo")
    is_payment_required: bool = Field(..., alias="isPaymentRequired")
    playback_position: float | None = Field(..., alias="playbackPosition")
    owner: Owner
    require_sensitive_masking: bool = Field(..., alias="requireSensitiveMasking")
    video_live: VideoLive | None = Field(..., alias="videoLive")
    is_muted: bool | None = Field(..., alias="isMuted")


class Tag(BaseModel):
    """A class that represents a tag."""

    name: str
    is_locked: bool = Field(..., alias="isLocked")
    is_locked_by_system: bool = Field(..., alias="isLockedBySystem")
    is_nicodic_article_exists: bool = Field(..., alias="isNicodicArticleExists")


MylistSortKey = Literal[
    "addedAt",
    "title",
    "mylistComment",
    "registeredAt",
    "viewCount",
    "lastComment",
    "commentCount",
    "likeCount",
    "mylistCount",
    "duration",
]


MylistSortOrder = Literal["desc", "asc"]


class MylistItem(BaseModel):
    """A class that represents a mylist item."""

    item_id: int = Field(..., alias="itemId")
    watch_id: str = Field(..., alias="watchId")
    description: str
    decorated_description_html: str = Field(..., alias="decoratedDescriptionHtml")
    added_at: str = Field(..., alias="addedAt")
    status: Literal["public", "hidden", "deleted"]
    video: EssentialVideo


class Mylist(BaseModel):
    """A class that represents a mylist."""

    id_: int = Field(..., alias="id")
    name: str
    description: str
    decorated_description_html: str = Field(..., alias="decoratedDescriptionHtml")
    default_sort_key: MylistSortKey = Field(..., alias="defaultSortKey")
    default_sort_order: MylistSortOrder = Field(..., alias="defaultSortOrder")
    items: list[MylistItem]
    total_item_count: int = Field(..., alias="totalItemCount")
    has_next: bool = Field(..., alias="hasNext")
    is_public: bool = Field(..., alias="isPublic")
    owner: Owner
    has_invisible_items: bool = Field(..., alias="hasInvisibleItems")
    follower_count: int = Field(..., alias="followerCount")
    is_following: bool = Field(..., alias="isFollowing")


class SeriesOwner(BaseModel):
    """A class that represents the owner of a series."""

    type_: Literal["user", "channel"] = Field(..., alias="type")
    id_: str = Field(..., alias="id")
    user: EssentialUser | None
    channel: EssentialChannel | None


class SeriesDetail(BaseModel):
    """A class that represents the detail of a series."""

    id_: int = Field(..., alias="id")
    owner: SeriesOwner
    title: str
    description: str
    decorated_description_html: str = Field(..., alias="decoratedDescriptionHtml")
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    is_listed: bool = Field(..., alias="isListed")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")


class SeriesVideoMeta(BaseModel):
    """A class that represents the metadata of a series video."""

    id_: str = Field(..., alias="id")
    order: int
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")


class SeriesItem(BaseModel):
    """A class that represents a series item."""

    meta: SeriesVideoMeta
    video: EssentialVideo


class HistoryItem(BaseModel):
    """A class that represents a history item."""

    frontend_id: int = Field(..., alias="frontendId")
    is_maybe_like_user_item: bool = Field(..., alias="isMaybeLikeUserItem")
    last_viewed_at: str = Field(..., alias="lastViewedAt")
    playback_position: float = Field(..., alias="playbackPosition")
    video: EssentialVideo
    views: int
    watch_id: str = Field(..., alias="watchId")



"""This module contains the class that represents a search channel object."""

from __future__ import annotations

from pydantic import BaseModel, Field


class ChSearchAPIMeta(BaseModel):
    """Represents the metadata of a channel API response."""

    status: int
    total: int | None = Field(None)
    count: int | None = Field(None)
    error_code: str | None = Field(None, alias="errorCode")
    error_message: str | None = Field(None, alias="errorMessage")
    error_details: dict[str, str] | None = Field(None, alias="errorDetails")


class ChSearchAPIResponse(BaseModel):
    """Represents a channel API response."""

    meta: ChSearchAPIMeta
    data: list[ChannelSearchItem] | None

class ChannelTag(BaseModel):
    """Represents a channel tag."""

    text: str


class ChannelCategory(BaseModel):
    """Represents a channel category."""

    category_id: int = Field(..., alias="categoryId")
    category_name: str = Field(..., alias="categoryName")
    category_top_page_url: str = Field(..., alias="categoryTopPageUrl")


class ChannelDetail(BaseModel):
    """Represents a channel detail."""

    tags: list[ChannelTag]
    category: ChannelCategory


class ChannelSearchItem(BaseModel):
    """Represents a channel search item."""

    id_: int = Field(..., alias="id")
    name: str
    description: str
    description_html: str | None = Field(None, alias="descriptionHtml")
    is_free: bool = Field(..., alias="isFree")
    screen_name: str = Field(..., alias="screenName")
    owner_name: str = Field(..., alias="ownerName")
    price: int
    body_price: int = Field(..., alias="bodyPrice")
    url: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    thumbnail_small_url: str = Field(..., alias="thumbnailSmallUrl")
    can_admit: bool = Field(..., alias="canAdmit")
    is_adult: bool = Field(..., alias="isAdult")
    detail: ChannelDetail



"""Decorators for the niconico package."""

from __future__ import annotations

from functools import wraps
from typing import TYPE_CHECKING, Any, Callable, TypeVar

from niconico.exceptions import LoginRequiredError, PremiumRequiredError

if TYPE_CHECKING:
    from niconico.base.client import BaseClient

F = TypeVar("F", bound=Callable[..., Any])


def login_required(*, premium: bool = False) -> Callable[[F], F]:
    """A decorator that requires a login to be performed."""

    def decorator(func: F) -> F:
        @wraps(func)
        def wrapper(self: BaseClient, *args: Any, **kwargs: Any) -> F:  # noqa: ANN401
            if not self.niconico.logined:
                raise LoginRequiredError(message="Login is required to use this function.")
            if premium and not self.niconico.premium:
                raise PremiumRequiredError(message="Premium account is required to use this function.")
            return func(self, *args, **kwargs)

        return wrapper  # type: ignore  # noqa: PGH003

    return decorator



"""A module that contains classes that represent responses from the NvAPI."""

from __future__ import annotations

from typing import Generic, TypeVar

from pydantic import BaseModel, Field

from niconico.objects.user import (
    NicoUser,
    OwnNicoUser,
    OwnVideoItem,
    OwnVideosLimitation,
    RelationshipUser,
    RelationshipUsersSummary,
    UserMylistItem,
    UserRelationships,
    UserSeriesItem,
    UserSeriesThumbnails,
    UserVideoItem,
)
from niconico.objects.user.search import UserSearchItem
from niconico.objects.video import EssentialVideo, HistoryItem, Mylist, SeriesDetail, SeriesItem, Tag
from niconico.objects.video.ranking import Genre
from niconico.objects.video.search import EssentialMylist, EssentialSeries, FacetItem, VideoSearchAdditionals

T = TypeVar("T")


class NvAPIMeta(BaseModel):
    """A class that represents the metadata of a response from the NvAPI."""

    status: int
    error_code: str | None = Field(None, alias="errorCode")


class NvAPIResponse(BaseModel, Generic[T]):
    """A class that represents a response from the NvAPI."""

    meta: NvAPIMeta
    data: T | None


class VideoItem(BaseModel):
    """A class that represents an item of a videos response from the NvAPI."""

    watch_id: str = Field(..., alias="watchId")
    video: EssentialVideo


class VideosData(BaseModel):
    """A class that represents the data of a videos response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/videos?watchIds=<video_id>
    """

    items: list[VideoItem]


class TagsData(BaseModel):
    """A class that represents the data of a tags response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/videos/<video_id>/tags
    """

    is_lockable: bool = Field(..., alias="isLockable")
    is_editable: bool = Field(..., alias="isEditable")
    uneditable_reason: str | None = Field(None, alias="uneditableReason")
    tags: list[Tag]


class MylistData(BaseModel):
    """A class that represents the data of a mylist response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/mylists/<mylist_id>
    """

    mylist: Mylist


class SeriesData(BaseModel):
    """A class that represents the data of a series response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/series/<series_id>
    """

    detail: SeriesDetail
    total_count: int = Field(..., alias="totalCount")
    items: list[SeriesItem]


class GenresData(BaseModel):
    """A class that represents the data of a genre response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/genres
    """

    genres: list[Genre]


class PopularTagsData(BaseModel):
    """A class that represents the data of a popular tags response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/genres/<genre_key>/popular-tags
    """

    start_at: str = Field(..., alias="startAt")
    tags: list[str]


class RankingData(BaseModel):
    """A class that represents the data of a ranking response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/ranking/genre/<genre_key>
    """

    items: list[EssentialVideo]
    has_next: bool = Field(..., alias="hasNext")


class VideoSearchData(BaseModel):
    """A class that represents the data of a video search response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/search/video
    """

    search_id: str = Field(..., alias="searchId")
    keyword: str | None
    tag: str | None
    genres: list[Genre]
    total_count: int = Field(..., alias="totalCount")
    has_next: bool = Field(..., alias="hasNext")
    items: list[EssentialVideo]
    additionals: VideoSearchAdditionals


class FacetData(BaseModel):
    """A class that represents the data of a facet response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/search/facet
    """

    items: list[FacetItem]


class ListSearchData(BaseModel):
    """A class that represents the data of a list search response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/search/list
    """

    search_id: str = Field(..., alias="searchId")
    total_count: int = Field(..., alias="totalCount")
    has_next: bool = Field(..., alias="hasNext")
    items: list[EssentialSeries | EssentialMylist]


class AccessRightsData(BaseModel):
    """A class that represents the data of an access rights response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/watch/<video_id>/access-rights/<type>
    """

    content_url: str | None = Field(None, alias="contentUrl")
    create_time: str | None = Field(None, alias="createTime")
    expire_time: str | None = Field(None, alias="expireTime")


class HistoryData(BaseModel):
    """A class that represents the data of a history response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/me/watch/history
    """

    items: list[HistoryItem]
    total_count: int = Field(..., alias="totalCount")


class UserData(BaseModel):
    """A class that represents the data of a user response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/<user_id>
    """

    user: NicoUser
    relationships: UserRelationships


class OwnUserData(BaseModel):
    """A class that represents the data of own user response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/me
    """

    user: OwnNicoUser


class RelationshipUsersData(BaseModel):
    """A class that represents the data of a relationship users response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/<user_id>/<type>/users
    """

    items: list[RelationshipUser]
    summary: RelationshipUsersSummary


class UserVideosData(BaseModel):
    """A class that represents the data of a user video response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v3/users/<user_id>/videos
    """

    items: list[UserVideoItem]
    total_count: int = Field(..., alias="totalCount")


class OwnVideosData(BaseModel):
    """A class that represents the data of own videos response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v2/users/me/videos
    """

    items: list[OwnVideoItem]
    total_count: int = Field(..., alias="totalCount")
    total_item_count: int = Field(..., alias="totalItemCount")
    limitation: OwnVideosLimitation


class UserMylistsData(BaseModel):
    """A class that represents the data of mylists response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/<user_id>/mylists
    """

    mylists: list[UserMylistItem]


class UserSeriesData(BaseModel):
    """A class that represents the data of a series response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/<user_id>/series
    """

    total_count: int = Field(..., alias="totalCount")
    items: list[UserSeriesItem]


class OwnSeriesData(BaseModel):
    """A class that represents the data of a series response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/users/me/series
    """

    total_count: int = Field(..., alias="totalCount")
    items: list[UserSeriesItem]
    thumbnails: UserSeriesThumbnails


class UserSearchData(BaseModel):
    """A class that represents the data of a list search response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/search/user
    """

    request_id: str = Field(..., alias="requestId")
    total_count: int = Field(..., alias="totalCount")
    has_next: bool = Field(..., alias="hasNext")
    items: list[UserSearchItem]


class ThreadKeyData(BaseModel):
    """A class that represents the data of a thread key response from the NvAPI.

    ref: https://nvapi.nicovideo.jp/v1/comment/keys/thread?videoId=<video_id>
    """

    thread_key: str = Field(..., alias="threadKey")



"""This module contains the class that represents common objects."""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field


class UserIcon(BaseModel):
    """A class that represents the icons of a user."""

    small: str
    large: str


class EssentialUser(BaseModel):
    """A class that represents an essential user object."""

    type_: Literal["essential"] = Field(..., alias="type")
    is_premium: bool = Field(..., alias="isPremium")
    description: str
    stripped_description: str = Field(..., alias="strippedDescription")
    short_description: str = Field(..., alias="shortDescription")
    id_: int = Field(..., alias="id")
    nickname: str
    icons: UserIcon


class EssentialChannel(BaseModel):
    """A class that represents a essential channel object."""

    id_: str = Field(..., alias="id")
    name: str
    description: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    thumbnail_small_url: str = Field(..., alias="thumbnailSmallUrl")



"""Module for channel search client."""

from __future__ import annotations

from typing import Literal

import requests

from niconico.base.client import BaseClient
from niconico.objects.channel.search import ChannelSearchItem, ChSearchAPIResponse


class ChannelSearchClient(BaseClient):
    """Client for channel search."""

    def search_channels(
        self,
        query: str,
        *,
        search_type: Literal["keyword", "tag"] = "keyword",
        limit: int = 20,
        offset: int = 0,
        order: Literal["desc", "asc"] = "desc",
        sort: Literal["updateTime"] | None = None,
    ) -> list[ChannelSearchItem]:
        """Search channels by query.

        Args:
            query (str): Query to search.
            search_type (Literal["keyword", "tag"], optional): Search type. Defaults to "keyword".
            limit (int, optional): Limit. Defaults to 20.
            offset (int, optional): Offset. Defaults to 0.
            order (Literal["desc", "asc"], optional): Order. Defaults to "desc".
            sort (Literal["updateTime"], optional): Sort. Default is created time.

        Returns:
            list[ChannelSearchItem]: Channel search item list.
        """
        query_dict = {
            "query": query,
            "searchType": search_type,
            "limit": str(limit),
            "offset": str(offset),
            "order": order,
            "responseGroup": "detail",
        }
        if sort is not None:
            query_dict["sort"] = sort
        query_str = "&".join(f"{key}={value}" for key, value in query_dict.items())
        res = self.niconico.get(f"https://public-api.ch.nicovideo.jp/v1/open/search/channels?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = ChSearchAPIResponse(**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return []



"""Data models for the watch API response."""

from __future__ import annotations

from typing import Any, Generic, TypeVar

from pydantic import BaseModel, Field

from niconico.objects.video import EssentialVideo


class WatchAPIMeta(BaseModel):
    """Meta data of the watch API response."""

    status: int
    code: str


class MetaTag(BaseModel):
    """Data model of a meta tag."""

    name: str | None = Field(None)
    property_: str | None = Field(None, alias="property")
    content: str


class LinkTag(BaseModel):
    """Data model of a link tag."""

    href: str
    rel: str
    attrs: dict[str, str] | list


class WatchPageMetadata(BaseModel):
    """Data model of the metadata of a watch page."""

    title: str
    meta_tags: list[MetaTag] = Field(..., alias="metaTags")
    link_tags: list[LinkTag] = Field(..., alias="linkTags")
    json_lds: Any = Field(..., alias="jsonLds")


class WatchChannelThumbnail(BaseModel):
    """Data model of the thumbnail of a watch channel."""

    url: str
    small_url: str | None = Field(..., alias="smallUrl")


class WatchChannelViewerFollow(BaseModel):
    """Data model of the viewer follow status of a watch channel."""

    is_followed: bool = Field(..., alias="isFollowed")
    is_bookmarked: bool = Field(..., alias="isBookmarked")
    token: str
    token_timestamp: int = Field(..., alias="tokenTimestamp")


class WatchChannelViewer(BaseModel):
    """Data model of the viewer of a watch channel."""

    follow: WatchChannelViewerFollow


class WatchChannel(BaseModel):
    """Data model of a watch channel."""

    id_: str = Field(..., alias="id")
    name: str
    is_official_anime: bool = Field(..., alias="isOfficialAnime")
    is_display_ad_banner: bool = Field(..., alias="isDisplayAdBanner")
    thumbnail: WatchChannelThumbnail
    viewer: WatchChannelViewer


class WatchClient(BaseModel):
    """Data model of the client of the watch API response."""

    nicosid: str | None
    watch_id: str = Field(..., alias="watchId")
    watch_track_id: str = Field(..., alias="watchTrackId")


class CommentThreadId(BaseModel):
    """Data model of a comment thread ID."""

    id_: int = Field(..., alias="id")
    fork: int
    fork_label: str = Field(..., alias="forkLabel")


class CommentLayer(BaseModel):
    """Data model of a comment layer."""

    index: int
    is_translucent: bool = Field(..., alias="isTranslucent")
    thread_ids: list[CommentThreadId] = Field(..., alias="threadIds")


class CommentThread(BaseModel):
    """Data model of a comment thread."""

    id_: int = Field(..., alias="id")
    fork: int
    fork_label: str = Field(..., alias="forkLabel")
    video_id: str = Field(..., alias="videoId")
    is_active: bool = Field(..., alias="isActive")
    is_default_post_target: bool = Field(..., alias="isDefaultPostTarget")
    is_easy_comment_post_target: bool = Field(..., alias="isEasyCommentPostTarget")
    is_leaf_required: bool = Field(..., alias="isLeafRequired")
    is_owner_thread: bool = Field(..., alias="isOwnerThread")
    is_threadkey_required: bool = Field(..., alias="isThreadkeyRequired")
    threadkey: str | None = Field(..., alias="threadkey")
    is_184_forced: bool = Field(..., alias="is184Forced")
    has_nicoscript: bool = Field(..., alias="hasNicoscript")
    label: str
    postkey_status: int = Field(..., alias="postkeyStatus")
    server: str


class CommentNgScore(BaseModel):
    """Data model of the score of a comment NG."""

    is_disabled: bool = Field(..., alias="isDisabled")


class CommentNgViewerItem(BaseModel):
    """Data model of an item of a comment NG viewer."""

    type_: str = Field(..., alias="type")
    source: str
    registered_at: str = Field(..., alias="registeredAt")


class CommentNgViewer(BaseModel):
    """Data model of a comment NG viewer."""

    revision: int
    count: int
    items: list[CommentNgViewerItem]


class CommentNg(BaseModel):
    """Data model of a comment NG."""

    ng_score: CommentNgScore = Field(..., alias="ngScore")
    channel: list
    owner: list
    viewer: CommentNgViewer | None


class NvCommentDataTarget(BaseModel):
    """Data model of a target of an NV comment data."""

    id_: str = Field(..., alias="id")
    fork: str


class NvCommentDataParams(BaseModel):
    """Data model of the parameters of an NV comment data."""

    targets: list[NvCommentDataTarget]
    language: str


class NvCommentData(BaseModel):
    """Data model of an NV comment data."""

    thread_key: str = Field(..., alias="threadKey")
    server: str
    params: NvCommentDataParams


class WatchComment(BaseModel):
    """Data model of the comment of the watch API response."""

    layers: list[CommentLayer]
    threads: list[CommentThread]
    ng: CommentNg
    is_attention_required: bool = Field(..., alias="isAttentionRequired")
    nv_comment: NvCommentData = Field(..., alias="nvComment")


class EasyCommentNicodic(BaseModel):
    """Data model of a Nicodic of an easy comment."""

    title: str
    view_title: str = Field(..., alias="viewTitle")
    summary: str
    link: str


class EasyCommentPhrase(BaseModel):
    """Data model of a phrase of an easy comment."""

    text: str
    nicodic: EasyCommentNicodic | None


class WatchEasyComment(BaseModel):
    """Data model of an easy comment of the watch API response."""

    phrases: list[EasyCommentPhrase]


class WatchGenre(BaseModel):
    """Data model of a genre of the watch API response."""

    key: str
    label: str
    is_immoral: bool = Field(..., alias="isImmoral")
    is_disabled: bool = Field(..., alias="isDisabled")
    is_not_set: bool = Field(..., alias="isNotSet")


class WatchMediaDomandVideo(BaseModel):
    """Data model of a video of a watch media DOMAND."""

    id_: str = Field(..., alias="id")
    is_available: bool = Field(..., alias="isAvailable")
    label: str
    bit_rate: int = Field(..., alias="bitRate")
    width: int
    height: int
    quality_level: int = Field(..., alias="qualityLevel")
    recommended_highest_audio_quality_level: int = Field(..., alias="recommendedHighestAudioQualityLevel")


class AudioLoudness(BaseModel):
    """Data model of the loudness of an audio."""

    type_: str = Field(..., alias="type")
    value: float


class WatchMediaDomandAudio(BaseModel):
    """Data model of an audio of a watch media DOMAND."""

    id_: str = Field(..., alias="id")
    is_available: bool = Field(..., alias="isAvailable")
    bit_rate: int = Field(..., alias="bitRate")
    sampling_rate: int = Field(..., alias="samplingRate")
    integrated_loudness: float = Field(..., alias="integratedLoudness")
    true_peak: float = Field(..., alias="truePeak")
    quality_level: int = Field(..., alias="qualityLevel")
    loudness_collection: list[AudioLoudness] = Field(..., alias="loudnessCollection")


class WatchMediaDomand(BaseModel):
    """Data model of a watch media DOMAND."""

    videos: list[WatchMediaDomandVideo]
    audios: list[WatchMediaDomandAudio]
    is_storyboard_available: bool = Field(..., alias="isStoryboardAvailable")
    access_right_key: str = Field(..., alias="accessRightKey")


class WatchMedia(BaseModel):
    """Data model of the media of the watch API response."""

    domand: WatchMediaDomand
    delivery: None
    delivery_legacy: None = Field(..., alias="deliveryLegacy")


class WatchOwnerViewer(BaseModel):
    """Data model of a viewer of a watch owner."""

    is_following: bool = Field(..., alias="isFollowing")


class WatchOwner(BaseModel):
    """Data model of a watch owner."""

    id_: int = Field(..., alias="id")
    nickname: str
    icon_url: str = Field(..., alias="iconUrl")
    channel: None
    live: None
    is_videos_public: bool = Field(..., alias="isVideosPublic")
    is_mylists_public: bool = Field(..., alias="isMylistsPublic")
    video_live_notice: None = Field(..., alias="videoLiveNotice")
    viewer: WatchOwnerViewer | None


class WatchPaymentVideo(BaseModel):
    """Data model of a video of a watch payment."""

    is_ppv: bool = Field(..., alias="isPpv")
    is_admission: bool = Field(..., alias="isAdmission")
    is_continuation_benefit: bool = Field(..., alias="isContinuationBenefit")
    is_premium: bool = Field(..., alias="isPremium")
    watchable_user_type: str = Field(..., alias="watchableUserType")
    commentable_user_type: str = Field(..., alias="commentableUserType")
    billing_type: str = Field(..., alias="billingType")


class WatchPaymentPreviewCheck(BaseModel):
    """Data model of a check of a watch payment preview."""

    is_enabled: bool = Field(..., alias="isEnabled")


class WatchPaymentPreview(BaseModel):
    """Data model of a preview of a watch payment."""

    ppv: WatchPaymentPreviewCheck
    admission: WatchPaymentPreviewCheck
    continuation_benefit: WatchPaymentPreviewCheck = Field(..., alias="continuationBenefit")
    premium: WatchPaymentPreviewCheck


class WatchPayment(BaseModel):
    """Data model of a payment of the watch API response."""

    video: WatchPaymentVideo
    preview: WatchPaymentPreview


class PlayerInitialPlayback(BaseModel):
    """Data model of the initial playback of a player."""

    type_: str = Field(..., alias="type")
    position_sec: float = Field(..., alias="positionSec")


class WatchPlayerComment(BaseModel):
    """Data model of a comment of a watch player."""

    is_default_invisible: bool = Field(..., alias="isDefaultInvisible")


class WatchPlayer(BaseModel):
    """Data model of a player of the watch API response."""

    initial_playback: PlayerInitialPlayback | None = Field(..., alias="initialPlayback")
    comment: WatchPlayerComment
    layer_mode: int = Field(..., alias="layerMode")


class RankingGenre(BaseModel):
    """Data model of a genre of a ranking."""

    rank: int
    genre: str
    date_time: str = Field(..., alias="dateTime")


class RankingTag(BaseModel):
    """Data model of a tag of a ranking."""

    tag: str
    regularized_tag: str = Field(..., alias="regularizedTag")
    rank: int
    genre: str
    date_time: str = Field(..., alias="dateTime")


class WatchRanking(BaseModel):
    """Data model of a ranking of the watch API response."""

    genre: RankingGenre | None
    popular_tag: list[RankingTag] = Field(..., alias="popularTag")


class WatchSeriesVideos(BaseModel):
    """Data model of the videos of a watch series."""

    prev: EssentialVideo | None
    next_: EssentialVideo | None = Field(..., alias="next")
    first: EssentialVideo | None


class WatchSeries(BaseModel):
    """Data model of a series of the watch API response."""

    id_: int = Field(..., alias="id")
    title: str
    description: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    video: WatchSeriesVideos


class WatchSystem(BaseModel):
    """Data model of the system of the watch API response."""

    server_time: str = Field(..., alias="serverTime")
    is_peak_time: bool = Field(..., alias="isPeakTime")
    is_stella_alive: bool = Field(..., alias="isStellaAlive")


class WatchTagEdit(BaseModel):
    """Data model of an edit of a watch tag."""

    is_editable: bool = Field(..., alias="isEditable")
    uneditable_reason: str | None = Field(..., alias="uneditableReason")
    edit_key: str | None = Field(..., alias="editKey")


class WatchTag(BaseModel):
    """Data model of a tag of the watch API response."""

    items: list
    has_r18_tag: bool = Field(..., alias="hasR18Tag")
    is_published_nicoscript: bool = Field(..., alias="isPublishedNicoscript")
    edit: WatchTagEdit
    viewer: WatchTagEdit | None


class WatchVideoCount(BaseModel):
    """Data model of the count of a watch video."""

    view: int
    comment: int
    mylist: int
    like: int


class WatchVideoThumbnail(BaseModel):
    """Data model of a thumbnail of a watch video."""

    url: str
    middle_url: str | None = Field(..., alias="middleUrl")
    large_url: str | None = Field(..., alias="largeUrl")
    player: str
    ogp: str


class WatchVideoRating(BaseModel):
    """Data model of the rating of a watch video."""

    is_adult: bool = Field(..., alias="isAdult")


class WatchVideoViewerLike(BaseModel):
    """Data model of the like status of a viewer of a watch video."""

    is_liked: bool = Field(..., alias="isLiked")
    count: int | None


class WatchVideoViewer(BaseModel):
    """Data model of a viewer of a watch video."""

    is_owner: bool = Field(..., alias="isOwner")
    like: WatchVideoViewerLike


class WatchVideo(BaseModel):
    """Data model of a video of the watch API response."""

    id_: str = Field(..., alias="id")
    title: str
    description: str
    count: WatchVideoCount
    duration: int
    thumbnail: WatchVideoThumbnail
    rating: WatchVideoRating
    registered_at: str = Field(..., alias="registeredAt")
    is_private: bool = Field(..., alias="isPrivate")
    is_deleted: bool = Field(..., alias="isDeleted")
    is_no_banner: bool = Field(..., alias="isNoBanner")
    is_authentication_required: bool = Field(..., alias="isAuthenticationRequired")
    is_embed_player_allowed: bool = Field(..., alias="isEmbedPlayerAllowed")
    is_gift_allowed: bool = Field(..., alias="isGiftAllowed")
    viewer: WatchVideoViewer | None
    watchable_user_type_for_payment: str = Field(..., alias="watchableUserTypeForPayment")
    commentable_user_type_for_payment: str = Field(..., alias="commentableUserTypeForPayment")


class WatchViewerExistence(BaseModel):
    """Data model of the existence of a watch viewer."""

    age: int
    prefecture: str
    sex: str


class WatchViewer(BaseModel):
    """Data model of a viewer of the watch API response."""

    id_: int = Field(..., alias="id")
    nickname: str
    is_premium: bool = Field(..., alias="isPremium")
    allow_sensitive_contents: bool = Field(..., alias="allowSensitiveContents")
    existence: WatchViewerExistence


class WatchVideoLive(BaseModel):
    """Data model of a live video of the watch API response."""

    program_id: str = Field(..., alias="programId")
    begin_at: str = Field(..., alias="beginAt")
    end_at: str = Field(..., alias="endAt")


class WatchData(BaseModel):
    """Data model of the data of the watch API response."""

    ads: None
    category: None
    channel: WatchChannel | None
    client: WatchClient
    comment: WatchComment
    community: None
    easy_comment: WatchEasyComment = Field(..., alias="easyComment")
    external: Any
    genre: WatchGenre
    marquee: Any
    media: WatchMedia
    ok_reason: str = Field(..., alias="okReason")
    owner: WatchOwner
    payment: WatchPayment
    pc_watch_page: Any = Field(..., alias="pcWatchPage")
    player: WatchPlayer
    ppv: None
    ranking: WatchRanking
    series: WatchSeries | None
    smartphone: None
    system: WatchSystem
    tag: WatchTag
    video: WatchVideo
    video_ads: Any = Field(..., alias="videoAds")
    video_live: WatchVideoLive | None = Field(..., alias="videoLive")
    viewer: WatchViewer | None
    waku: Any


class WatchResponseError(BaseModel):
    """Data model of the error response of the watch API."""

    is_custom_error: bool = Field(..., alias="isCustomError")
    status_code: int = Field(..., alias="statusCode")
    error_code: str = Field(..., alias="errorCode")
    reason_code: str | None = Field(..., alias="reasonCode")
    deleted_message: str | None = Field(..., alias="deletedMessage")
    community_link: str | None = Field(..., alias="communityLink")
    publish_scheduled_at: str | None = Field(..., alias="publishScheduledAt")
    data: None


class WatchAPIData(BaseModel):
    """Data model of the data of the watch API response."""

    metadata: WatchPageMetadata
    google_tag_manager: Any | None = Field(..., alias="googleTagManager")
    response: WatchData


class WatchAPIErrorData(BaseModel):
    """Data model of an error response of the watch API."""

    metadata: WatchPageMetadata
    response: WatchResponseError


T = TypeVar("T")


class WatchAPIResponse(BaseModel, Generic[T]):
    """Data model of the watch API response.

    ref: https://www.nicovideo.jp/watch/<video_id>?responseType=json
    """

    meta: WatchAPIMeta
    data: T


class StoryboardImage(BaseModel):
    """Data model of an image of a storyboard."""

    timestamp: int
    url: str


class StoryboardResponse(BaseModel):
    """Data model of the storyboard response.

    ref: https://asset.domand.nicovideo.jp/<id>/storyboard/~~~
    """

    columns: int
    images: list[StoryboardImage]
    interval: int
    rows: int
    thumbnail_height: int = Field(..., alias="thumbnailHeight")
    thumbnail_width: int = Field(..., alias="thumbnailWidth")
    version: str


class NvCommentAPIMeta(BaseModel):
    """Data model of the NV comment API meta."""

    status: int
    error_code: str | None = Field(None, alias="errorCode")


class GlobalComment(BaseModel):
    """Data model of a global comment."""

    id_: str = Field(..., alias="id")
    count: int


class Comment(BaseModel):
    """Data model of a comment."""

    body: str
    commands: list[str]
    id_: str = Field(..., alias="id")
    is_my_post: bool = Field(..., alias="isMyPost")
    is_premium: bool = Field(..., alias="isPremium")
    nicoru_count: int = Field(..., alias="nicoruCount")
    nicoru_id: str | None = Field(None, alias="nicoruId")
    no: int
    posted_at: str = Field(..., alias="postedAt")
    score: int
    source: str
    user_id: str = Field(..., alias="userId")
    vpos_ms: int = Field(..., alias="vposMs")


class NvCommentThread(BaseModel):
    """Data model of an NV comment thread."""

    id_: str = Field(..., alias="id")
    fork: str
    comment_count: int = Field(..., alias="commentCount")
    comments: list[Comment]


class NvCommentAPIData(BaseModel):
    """Data model of the NV comment data."""

    global_comments: list[GlobalComment] = Field(..., alias="globalComments")
    threads: list[NvCommentThread]


class NvCommentAPIResponse(BaseModel):
    """Data model of the NV comment API response.

    ref: https://public.nvcomment.nicovideo.jp/v1/threads
    """

    data: NvCommentAPIData | None
    meta: NvCommentAPIMeta



"""This module contains the class that represents a channel object."""

from __future__ import annotations

from typing import Generic, TypeVar

from pydantic import BaseModel, Field

T = TypeVar("T")


class ChAPIMeta(BaseModel):
    """Represents the metadata of a channel API response."""

    status: int
    error_code: str | None = Field(None, alias="errorCode")
    error_message: str | None = Field(None, alias="errorMessage")
    error_details: dict[str, str] | None = Field(None, alias="errorDetails")


class ChAPIResponse(BaseModel, Generic[T]):
    """Represents a channel API response."""

    meta: ChAPIMeta
    data: T | None


class ChannelStatus(BaseModel):
    """Represents the status of a channel API response."""

    is_open: bool = Field(..., alias="isOpen")
    is_admission_available : bool = Field(..., alias="isAdmissionAvailable")
    is_adult_channel: bool = Field(..., alias="isAdultChannel")
    is_gravure_channel: bool = Field(..., alias="isGravureChannel")


class ChannelCategory(BaseModel):
    """Represents the category of a channel API response."""

    id_: int = Field(..., alias="id")
    name: str


class ChannelBasicConfiguration(BaseModel):
    """Represents the basic configuration of a channel API response."""

    name: str
    description: str
    screen_name: str = Field(..., alias="screenName")
    category: ChannelCategory
    owner_name: str = Field(..., alias="ownerName")


class ChannelExposureConfiguration(BaseModel):
    """Represents the exposure configuration of a channel API response."""

    open_time: str | None = Field(..., alias="openTime")
    close_time: str | None = Field(..., alias="closeTime")


class ChannelAdmissionConfiguration(BaseModel):
    """Represents the admission configuration of a channel API response."""

    is_free: bool = Field(..., alias="isFree")
    is_first_month_free: bool = Field(..., alias="isFirstMonthFree")


class ChannelOthersConfiguration(BaseModel):
    """Represents the others configuration of a channel API response."""

    checkpoint_sales_enabled_flag: bool = Field(..., alias="checkpointSalesEnabledFlag")


class ChannelConfiguration(BaseModel):
    """Represents the configuration of a channel API response."""

    basic: ChannelBasicConfiguration
    exposure: ChannelExposureConfiguration
    admission: ChannelAdmissionConfiguration
    others: ChannelOthersConfiguration


class ChannelLocation(BaseModel):
    """Represents the location of a channel API response."""

    url: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    thumbnail_small_url: str = Field(..., alias="thumbnailSmallUrl")
    category_top_page_url: str = Field(..., alias="categoryTopPageUrl")


class ChannelData(BaseModel):
    """Represents the data of a channel API response."""

    id_: int = Field(..., alias="id")
    status: ChannelStatus
    configuration: ChannelConfiguration
    location: ChannelLocation



"""This module contains the class that represents a user object."""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field

from niconico.objects.common import EssentialChannel, UserIcon
from niconico.objects.video import EssentialVideo, MylistItem, MylistSortKey, MylistSortOrder, Owner


class SessionRelationships(BaseModel):
    """A class that represents the relationships of a user in a session."""

    is_following: bool = Field(..., alias="isFollowing")


class UserRelationships(BaseModel):
    """A class that represents the relationships of a user."""

    session_user: SessionRelationships = Field(..., alias="sessionUser")
    is_me: bool = Field(0, alias="isMe")


class UserLevel(BaseModel):
    """A class that represents the level of a user."""

    current_level: int = Field(..., alias="currentLevel")
    next_level_threshold_experience: int = Field(..., alias="nextLevelThresholdExperience")
    next_level_experience: int = Field(..., alias="nextLevelExperience")
    current_level_experience: int = Field(..., alias="currentLevelExperience")


class UserSNS(BaseModel):
    """A class that represents the SNS of a user."""

    type_: Literal["twitter", "instagram", "youtube", "facebook"] = Field(..., alias="type")
    label: str
    icon_url: str = Field(..., alias="iconUrl")
    screen_name: str = Field(..., alias="screenName")
    url: str


class UserCoverImage(BaseModel):
    """A class that represents the cover image of a user."""

    ogp_url: str = Field(..., alias="ogpUrl")
    pc_url: str = Field(..., alias="pcUrl")
    smartphone_url: str = Field(..., alias="smartphoneUrl")


class NicoUser(BaseModel):
    """A class that represents a user object."""

    description: str
    decorated_description_html: str = Field(..., alias="decoratedDescriptionHtml")
    stripped_description: str = Field(..., alias="strippedDescription")
    is_premium: bool = Field(..., alias="isPremium")
    registered_version: str = Field(..., alias="registeredVersion")
    followee_count: int = Field(..., alias="followeeCount")
    follower_count: int = Field(..., alias="followerCount")
    user_level: UserLevel = Field(..., alias="userLevel")
    user_channel: EssentialChannel | None = Field(..., alias="userChannel")
    is_nicorepo_readable: bool = Field(..., alias="isNicorepoReadable")
    sns: list[UserSNS] = Field(..., alias="sns")
    cover_image: UserCoverImage | None = Field(..., alias="coverImage")
    id_: int = Field(..., alias="id")
    nickname: str
    icons: UserIcon


class OwnNicoUser(NicoUser):
    """A class that represents the user object of the own user."""

    niconico_point: int = Field(..., alias="niconicoPoint")
    language: str
    premium_ticket_expire_time: str | None = Field(..., alias="premiumTicketExpireTime")
    creator_patronizing_score: int = Field(..., alias="creatorPatronizingScore")
    is_mail_bounced: bool = Field(..., alias="isMailBounced")
    is_nicorepo_auto_posted_to_twitter: bool = Field(..., alias="isNicorepoAutoPostedToTwitter")


class RelationshipUser(BaseModel):
    """A class that represents a relationship user object."""

    type_: Literal["relationship"] = Field(..., alias="type")
    relationships: UserRelationships
    is_premium: bool = Field(..., alias="isPremium")
    description: str
    stripped_description: str = Field(..., alias="strippedDescription")
    short_description: str = Field(..., alias="shortDescription")
    id_: int = Field(..., alias="id")
    nickname: str
    icons: UserIcon


class RelationshipUsersSummary(BaseModel):
    """A class that represents the summary of a relationship users response from the NvAPI."""

    followees: int
    followers: int
    has_next: bool = Field(..., alias="hasNext")
    cursor: str


class VideoItemSeries(BaseModel):
    """A class that represents a series of a video item."""

    id_: int = Field(..., alias="id")
    title: str
    order: int


class UserVideoItem(BaseModel):
    """A class that represents a video item of a user."""

    series: VideoItemSeries | None
    essential: EssentialVideo


UserVideosSortKey = Literal[
    "registeredAt",
    "viewCount",
    "lastCommentTime",
    "commentCount",
    "likeCount",
    "mylistCount",
    "duration",
]

UserVideosSortOrder = Literal["asc", "desc"]


class OwnVideosLimitationUser(BaseModel):
    """A class that represents the user of the limitation of own videos."""

    uploadable_count: int | None = Field(None, alias="uploadableCount")
    uploaded_count_for_limitation: int | None = Field(None, alias="uploadedCountForLimitation")


class OwnVideosLimitation(BaseModel):
    """A class that represents the limitation of own videos."""

    border_id: int = Field(..., alias="borderId")
    user: OwnVideosLimitationUser


class OwnVideoItem(BaseModel):
    """A class that represents a video item of own videos."""

    is_capture_tweet_allowed: bool = Field(..., alias="isCaptureTweetAllowed")
    is_clip_tweet_allowed: bool = Field(..., alias="isClipTweetAllowed")
    is_community_member_only: bool = Field(..., alias="isCommunityMemberOnly")
    description: str
    is_hidden: bool = Field(..., alias="isHidden")
    is_deleted: bool = Field(..., alias="isDeleted")
    is_cpp_registered: bool = Field(..., alias="isCppRegistered")
    is_contents_tree_exists: bool = Field(..., alias="isContentsTreeExists")
    publish_timer_detail: str | None = Field(..., alias="publishTimerDetail")
    auto_delete_detail: str | None = Field(..., alias="autoDeleteDetail")
    is_exclude_from_upload_list: bool = Field(..., alias="isExcludeFromUploadList")
    like_count: int = Field(..., alias="likeCount")
    gift_point: int = Field(..., alias="giftPoint")
    essential: EssentialVideo
    series: VideoItemSeries | None


class UserMylistItem(BaseModel):
    """A class that represents a mylist item of a user."""

    id_: int = Field(..., alias="id")
    is_public: bool = Field(..., alias="isPublic")
    name: str
    description: str
    decorated_description_html: str = Field(..., alias="decoratedDescriptionHtml")
    default_sort_key: MylistSortKey = Field(..., alias="defaultSortKey")
    default_sort_order: MylistSortOrder = Field(..., alias="defaultSortOrder")
    items_count: int = Field(..., alias="itemsCount")
    owner: Owner
    sample_items: list[MylistItem]
    follower_count: int = Field(..., alias="followerCount")
    created_at: str = Field(..., alias="createdAt")
    is_following: bool = Field(..., alias="isFollowing")


class UserSeriesOwner(BaseModel):
    """A class that represents the owner of a user series item."""

    type_: Literal["user"] = Field(..., alias="type")
    id_: str = Field(..., alias="id")


class UserSeriesItem(BaseModel):
    """A class that represents a series item of a user."""

    id_: int = Field(..., alias="id")
    owner: UserSeriesOwner
    title: str
    is_listed: bool = Field(..., alias="isListed")
    description: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    items_count: int = Field(..., alias="itemsCount")


class UserSeriesThumbnails(BaseModel):
    """A class that represents the thumbnails of a user series."""

    default: str
    default_for_owner: str = Field(..., alias="defaultForOwner")



"""This module contains the base client class."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from niconico.niconico import NicoNico


class BaseClient:
    """A class that represents a base client."""

    niconico: NicoNico

    def __init__(self, niconico: NicoNico) -> None:
        """Initialize the base client."""
        self.niconico = niconico

    def log(self, type_: str, message: str) -> None:
        """Log a message."""
        return getattr(self.niconico.logger, type_)(message)



"""This module provides a class that represents a video ranking client."""

from __future__ import annotations

from typing import TYPE_CHECKING, Literal

import requests

from niconico.base.client import BaseClient
from niconico.objects.nvapi import GenresData, NvAPIResponse, PopularTagsData, RankingData

if TYPE_CHECKING:
    from niconico.objects.video.ranking import Genre


class VideoRankingClient(BaseClient):
    """A class that represents a video ranking client."""

    def get_genres(self) -> list[Genre]:
        """Get genres.

        Returns:
            list[Genre]: A list of available genres.
        """
        res = self.niconico.get("https://nvapi.nicovideo.jp/v2/genres")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[GenresData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.genres
        return []

    def get_popular_tags(self, genre_key: str) -> list[str]:
        """Get popular tags of a genre.

        Args:
            genre_key (str): The key of the genre.

        Returns:
            list[str]: A list of popular tags of the genre.
        """
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/genres/{genre_key}/popular-tags")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[PopularTagsData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.tags
        return []

    def get_ranking(
        self,
        genre_key: str,
        term: Literal["hour", "24h", "week", "month", "total"],
        *,
        page_size: Literal[25, 100] = 100,
        page: int = 1,
        tag: str | None = None,
        sensitive_contents: Literal["mask", "filter"] | None = None,
    ) -> RankingData | None:
        """Get a ranking.

        Args:
            genre_key (str): The key of the genre.
            term (Literal["hour", "24h", "week", "month", "total"]): The term of the ranking.
            page_size (int, optional): The size of the page. Defaults to 10.
            page (int, optional): The page number. Defaults to 1.
            tag (str, optional): The tag. Defaults to None.
            sensitive_contents (Literal["mask", "filter"], optional): The sensitive contents. Defaults to None.

        Returns:
            RankingData | None: The ranking data.
        """
        query = {
            "term": term,
            "pageSize": str(page_size),
            "page": str(page),
        }
        if tag is not None:
            query["tag"] = tag
        if sensitive_contents is not None:
            query["sensitiveContents"] = sensitive_contents
        query_str = "&".join([f"{key}={value}" for key, value in query.items()])
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/ranking/genre/{genre_key}?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[RankingData](**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return None

    def get_hot_topics(
        self,
        term: Literal["hour", "24h", "week", "month", "total"],
        *,
        page_size: Literal[25, 100] = 100,
        page: int = 1,
        sensitive_contents: Literal["mask", "filter"] | None = None,
    ) -> RankingData | None:
        """Get hot topics.

        Args:
            term (Literal["hour", "24h", "week", "month", "total"]): The term of the hot topics.
            page_size (int, optional): The size of the page. Defaults to 10.
            page (int, optional): The page number. Defaults to 1.
            sensitive_contents (Literal["mask", "filter"], optional): The sensitive contents. Defaults to None.

        Returns:
            list[str]: A list of hot topics.
        """
        query = {
            "term": term,
            "pageSize": str(page_size),
            "page": str(page),
        }
        if sensitive_contents is not None:
            query["sensitiveContents"] = sensitive_contents
        query_str = "&".join([f"{key}={value}" for key, value in query.items()])
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/ranking/hot-topic?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[RankingData](**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return None



"""This module provides a class that represents a video client."""

from __future__ import annotations

from typing import TYPE_CHECKING

import requests

from niconico.base.client import BaseClient
from niconico.decorators import login_required
from niconico.objects.nvapi import HistoryData, MylistData, NvAPIResponse, SeriesData, TagsData, VideosData
from niconico.video.ranking import VideoRankingClient
from niconico.video.search import VideoSearchClient
from niconico.video.watch import VideoWatchClient

if TYPE_CHECKING:
    from niconico.niconico import NicoNico
    from niconico.objects.video import EssentialVideo, Mylist, MylistSortKey, MylistSortOrder, Tag


class VideoClient(BaseClient):
    """A class that represents a video client."""

    ranking: VideoRankingClient
    search: VideoSearchClient
    watch: VideoWatchClient

    def __init__(self, niconico: NicoNico) -> None:
        """Initialize the client."""
        super().__init__(niconico)
        self.ranking = VideoRankingClient(niconico)
        self.search = VideoSearchClient(niconico)
        self.watch = VideoWatchClient(niconico)

    def get_video(self, video_id: str) -> EssentialVideo | None:
        """Get a video by its ID.

        Args:
            video_id (str): The ID of the video.

        Returns:
            EssentialVideo | None: The video object if found, None otherwise.
        """
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/videos?watchIds={video_id}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[VideosData](**res.json())
            if res_cls.data is not None and len(res_cls.data.items) >= 1:
                return res_cls.data.items[0].video
        return None

    def get_video_tags(self, video_id: str) -> list[Tag] | None:
        """Get the tags of a video by its ID.

        Args:
            video_id (str): The ID of the video.

        Returns:
            list[Tag] | None: The tags of the video if found, None otherwise.
        """
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/videos/{video_id}/tags")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[TagsData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.tags
        return None

    def get_mylist(
        self,
        mylist_id: str,
        *,
        page_size: int = 20,
        page: int = 1,
        sort_key: MylistSortKey | None = None,
        sort_order: MylistSortOrder | None = None,
    ) -> Mylist | None:
        """Get a mylist by its ID.

        Args:
            mylist_id (str): The ID of the mylist.
            page_size (int): The number of videos to get per page.
            page (int): The page number.
            sort_key (MylistSortKey | None): The sort key.
            sort_order (MylistSortOrder | None): The sort order.

        Returns:
            Mylist | None: The mylist object if found, None otherwise.
        """
        query = {"pageSize": str(page_size), "page": str(page)}
        if sort_key is not None:
            query["sortKey"] = sort_key
        if sort_order is not None:
            query["sortOrder"] = sort_order
        query_str = "&".join([f"{key}={value}" for key, value in query.items()])
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v2/mylists/{mylist_id}?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[MylistData](**res.json())
            if res_cls.data is not None:
                return res_cls.data.mylist
        return None

    def get_series(self, series_id: str, *, page_size: int = 100, page: int = 1) -> SeriesData | None:
        """Get a series by its ID.

        Args:
            series_id (str): The ID of the series.
            page_size (int): The number of videos to get per page.
            page (int): The page number.

        Returns:
            SeriesData | None: The series object if found, None otherwise.
        """
        query = {"pageSize": str(page_size), "page": str(page)}
        query_str = "&".join([f"{key}={value}" for key, value in query.items()])
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/series/{series_id}?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[SeriesData](**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return None

    @login_required()
    def get_history(self, *, page_size: int = 100, page: int = 1) -> HistoryData | None:
        """Get the history of the authenticated user.

        Args:
            page_size (int): The number of videos to get per page.
            page (int): The page number.

        Returns:
            HistoryData | None: The history data if successful, None otherwise.
        """
        query = {"pageSize": str(page_size), "page": str(page)}
        query_str = "&".join([f"{key}={value}" for key, value in query.items()])
        res = self.niconico.get(f"https://nvapi.nicovideo.jp/v1/users/me/watch/history?{query_str}")
        if res.status_code == requests.codes.ok:
            res_cls = NvAPIResponse[HistoryData](**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return None



"""This module provides a class that represents a channel client."""

from __future__ import annotations

from typing import TYPE_CHECKING

import requests

from niconico.base.client import BaseClient
from niconico.channel.search import ChannelSearchClient
from niconico.objects.channel import ChannelData, ChAPIResponse

if TYPE_CHECKING:
    from niconico.niconico import NicoNico


class ChannelClient(BaseClient):
    """A client that represents a channel client."""

    search: ChannelSearchClient

    def __init__(self, niconico: NicoNico) -> None:
        """Initialize the client."""
        super().__init__(niconico)
        self.search = ChannelSearchClient(niconico)

    def get_channel(self, channel_id: str) -> ChannelData | None:
        """Get a channel."""
        channel_id = channel_id.replace("ch", "")
        res = self.niconico.get(f"https://public-api.ch.nicovideo.jp/v2/open/channels/{channel_id}")
        if res.status_code == requests.codes.ok:
            res_cls = ChAPIResponse[ChannelData](**res.json())
            if res_cls.data is not None:
                return res_cls.data
        return None



"""This module provides classes that represent search objects."""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field

from niconico.objects.video import Owner
from niconico.objects.video.ranking import Genre

VideoSearchSortKey = Literal[
    "registeredAt",
    "viewCount",
    "lastCommentTime",
    "commentCount",
    "likeCount",
    "mylistCount",
    "duration",
    "hot",
    "personalized",
]

VideoSearchSortOrder = Literal["desc", "asc", "none"]


class RelatedTag(BaseModel):
    """A class that represents a related tag."""

    text: str
    type_: str = Field(..., alias="type")


class VideoSearchAdditionals(BaseModel):
    """A class that represents additional parameters for a video search."""

    tags: list[RelatedTag]


class FacetItem(BaseModel):
    """A class that represents a facet item."""

    genre: Genre
    count: int


ListType = Literal["mylist", "series"]


ListSearchSortKey = Literal["_hotTotalScore", "videoCount", "startTime"]


class EssentialSeries(BaseModel):
    """A class that represents an essential series."""

    id_: int = Field(..., alias="id")
    type_: Literal["series"] = Field(..., alias="type")
    title: str
    description: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    video_count: int = Field(..., alias="videoCount")
    owner: Owner
    is_muted: bool = Field(..., alias="isMuted")
    is_following: bool = Field(..., alias="isFollowing")
    follower_count: int = Field(..., alias="followerCount")


class EssentialMylist(BaseModel):
    """A class that represents an essential mylist."""

    id_: int = Field(..., alias="id")
    type_: Literal["mylist"] = Field(..., alias="type")
    title: str
    description: str
    thumbnail_url: str = Field(..., alias="thumbnailUrl")
    video_count: int = Field(..., alias="videoCount")
    owner: Owner
    is_muted: bool = Field(..., alias="isMuted")
    is_following: bool = Field(..., alias="isFollowing")
    follower_count: int = Field(..., alias="followerCount")
